local mod = HudMap:NewModule("Encounters", "AceEvent-3.0")

local CallbackHandler = LibStub:GetLibrary("CallbackHandler-1.0")
local callbacks = {}

local function groupIter(state, index)
	if index < 0 then return end
	local raid, party = GetNumRaidMembers(), GetNumPartyMembers()
	local prefix = raid > 0 and "raid" or "party"
	local unit = prefix .. index
	if UnitExists(unit) then
		return index + 1, unit
	elseif raid == 0 then
		return -1, "player"
	end
end

local function group()
	return groupIter, nil, 1
end

mod.group = group

function mod:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")	
end

function mod:COMBAT_LOG_EVENT_UNFILTERED(ev, timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellID, ...)
	local ev = callbacks[event]
	if ev then
		for _, module in ipairs(ev) do
			module[event](module, spellID, sourceName, destName, sourceGUID, destGUID)
		end
	end
end

function mod:RegisterModule(module)
	if module.registerOn then
		for _, e in ipairs(module.registerOn) do
			callbacks[e] = callbacks[e] or {}
			tinsert(callbacks[e], module)
		end
	end
end

local approximateUnits = {}
function mod:ApproximateBossPosition()
	wipe(approximateUnits)
	local prefix, max
	if GetNumRaidMembers() > 0 then
		prefix = "party"
		max = "4"
	elseif GetNumPartyMembers() > 0 then		
		prefix = "raid"
		max = 25
	end
	for i = 1, max do
		local _, cls = UnitClass(prefix..i)
		if not cls then break end
		
		if cls == "HUNTER" and UnitExists(prefix..i.."pet") then
			tinsert(approximateUnits, prefix..i)
		elseif cls == "ROGUE" or cls == "WARRIOR" or cls == "DEATHKNIGHT" then
			tinsert(approximateUnits, prefix..i)
		end
	end
	local sumX, sumY, sumN = 0, 0, 0
	for _, v in ipairs(approximateUnits) do
		local x, y = HudMap:GetUnitPosition(v)
		if x and y and x ~= 0 and y ~= 0 then
			sumX = sumX + x
			sumY = sumY + y
			sumN = sumN + 1
		end
	end
	return sumX / sumN, sumY / sumN
end

local GuidToMobID = setmetatable({}, {__index = function(t, k)
	local id = (k and tonumber(k:sub(9, 12), 16)) or 0
	rawset(t, k, id)
	return id	
end})

function mod:IsHeroic()
	local diff, switchable = select(6, GetInstanceInfo())
	if switchable then
		return diff == 1
	else
		return GetInstanceDifficulty() >= 3
	end
end

do
	function mod:RangeCheckAll(tex, radius, duration, nr, ng, nb, na, ar, ag, ab, aa, excludePlayer)
		for index, unit in group() do
			local n = UnitName(unit)
			if n and not UnitIsDead(unit) and not UnitIsUnit("player", unit) then
				local c = HudMap:PlaceRangeMarkerOnPartyMember(tex, n, radius, duration, nr, ng, nb, na):SetAlertColor(ar, ag, ab, aa):Appear():Rotate(360, duration)
				c.RegisterCallback(self, "Update", "UpdateRanges")
			end
		end	
	end
	
	function mod:RangeCheckSelf(tex, radius, duration, nr, ng, nb, na, ar, ag, ab, aa)
		local c = HudMap:PlaceRangeMarkerOnPartyMember(tex, "player", 12, 5, nr, ng, nb, na):SetAlertColor(ar, ag, ab, aa):Appear():Rotate(360, duration)
		c.RegisterCallback(self, "Update", "UpdateRangeToSelf")
	end

	function mod:UpdateRanges(cbk, marker, x, y, radius)
		if not marker.follow or not marker.radius then return end
		marker:Alert(HudMap:UnitDistance("player", marker.follow) < marker.radius)
	end
	
	function mod:UpdateRangeToSelf(cbk, marker, x, y, radius)
		if not marker.radius then return end
		
		local alert = false
		for index, unit in group() do
			if not UnitIsUnit("player", unit) and HudMap:UnitDistance("player", unit) < marker.radius then
				alert = true
				break
			end
		end
		marker:Alert(alert)
	end
end

function mod:HomeToTarget(unit, radius, r, g, b, a)
	radius = radius or 2
	r = r or 1
	g = g or 1
	b = b or 1
	a = a or 1	
end

function mod:GetMobTarget(mobID)
	for index, unit in group() do
		local g = UnitGUID(unit.."target")
		if g then
			if GuidToMobID[g] == mobID then
				return UnitName(unit.."targettarget")
			end
		end
	end	
end

local delays = {}
function mod:Delay(func, delay)
	delays[func] = GetTime() + delay
end

local timerFrame = CreateFrame("Frame", nil, UIParent)
timerFrame:SetWidth(1)
timerFrame:SetHeight(1)
timerFrame:SetScript("OnUpdate", function(self, t)
	local ct = GetTime()
	for k, v in pairs(delays) do
		if ct > v then
			k()
			delays[k] = nil
		end
	end
end)